/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Properties;
import java.util.logging.Logger;
import java.util.logging.Level;

import com.funambol.framework.core.PropParam;
import com.funambol.framework.core.Property;
import com.funambol.framework.filter.*;
import com.funambol.framework.logging.Sync4jLogger;

import java.io.*;

/**
 * @version $Id: SyncSourceService.java,v 1.1 2006/08/09 13:12:07 adeac Exp $
 */
public class SyncSourceService {

    //--------------------------  Constants  -------------------------

    private static final String PROPERTIES_FILE_NAME = "./../webapps/example-ws/config.ini";

    private static final String DEFAULD_DB_LOCATION = "./../webapps/example-ws/db/";
    private static final String DEFAULD_FORMAT = "";
    private static final String DEFAULD_TYPE = "sif-c";

    private static final String PROPERTY_DB_LOCATION = "location";
    private static final String PROPERTY_FORMAT = "format";
    private static final String PROPERTY_TYPE = "type";

    private static final String SYNCHRONIZED_ITEMS_LOCATION = "synchronized/";
    private static final String NEW_ITEMS_LOCATION = "new/";
    private static final String UPDATED_ITEMS_LOCATION = "updated/";
    private static final String DELETED_ITEMS_LOCATION = "deleted/";

    //--------------------------  Properties  ------------------------
    private Logger log = null;

    private Hashtable synchronizedItems = null;
    private Hashtable newItems = null;
    private Hashtable updatedItems = null;
    private Hashtable deletedItems = null;

    private String dbLocation = null;
    private String format = null;
    private String type = null;

    private static int id = 0;
    private String sessionId = null;

    //------------------------- Constructor  --------------------------

    public SyncSourceService() {

        log = Sync4jLogger.getLogger("ws");
        log.setLevel(Level.FINEST);

        log.finest("Creating SyncSourceService");

    }

    //------------------------- Public Methods  --------------------------

    public String beginSync(Long principalId,
                            Integer syncMode,
                            FilterClause filterClause) {

        String filter = new String("Empty");
        LogicalClause logical = null;
        Clause[] clauses;
        WhereClause wc = null;
        FieldClause fc = null;
        IDClause idc = null;
        Property p = null;

        if (filterClause != null)
            logical = filterClause.getClause();

        if (logical != null) {

            clauses = logical.getOperands();


            for (int i = 0; i < clauses.length; i++) {
                if (clauses[i] instanceof WhereClause)
                    wc = (WhereClause) clauses[i];
                else if (clauses[i] instanceof FieldClause)
                    fc = (FieldClause) clauses[i];
                else if (clauses[i] instanceof IDClause)
                    idc = (IDClause) clauses[i];
            }
        }
        if (fc != null)
            p = fc.getProperty();

        if (p != null) {
            PropParam[] pp = (PropParam[]) p.getPropParams().toArray(new PropParam[0]);
            String[] ve = (String[]) pp[0].getValEnums().toArray(new String[0]);
            filter = "Field filter: property name is " + p.getDisplayName() +
                    "and one of its paramiters is " + ve[0];
        }

        log.finest("beginSync method as been invoked. "
                + "Input parameters are: "
                + "PrincipalId = " + principalId
                + ", SyncMode = " + syncMode
                + ", FilterClause = " + filter);


        createNewSession();

        log.finest("session id = " + sessionId + " will be returned.");

        loadData();

        return sessionId;

    }

    public String beginSync(Long principalId, Integer syncMode) {


        log.finest("beginSync method as been invoked. "
                + "Input parameters are: "
                + "PrincipalId = " + principalId
                + ", SyncMode = " + syncMode);

        createNewSession();


        log.finest("session id = " + sessionId + " will be returned.");

        loadData();

        return sessionId;

    }

    public Item getItemFromId(String sessionId, String itemKey) {

        log.finest("getItemFromId method as been invoked. "
                + "Input parameters are: "
                + "SessionId = " + sessionId
                + ", ItemKey = " + itemKey);

        Item item = findItem(itemKey);

        log.finest("item with key = " + item.getKey() + " will be returned.");

        return item;

    }

    public String[] getAllItemKeys(String sessionId) {

        log.finest("getAllItemKeys method as been invoked. "
                + "Input parameter is: "
                + "SessionId = " + sessionId);

        String keys[] = getAllItemKeys();
        log.finest("keys = " + keys + " will be returned.");

        return keys;

    }


    public String[] getNewItemKeys(String sessionId, String since
            , String until) {


        log.finest("getNewItemKeys method as been invoked. "
                + "Input parameters are: "
                + "SessionId = " + sessionId
                + ", Since = " + since
                + ", Until = " + until);

        String keys[] = getItemKeys(newItems);
        log.finest("keys = " + keys + " will be returned.");

        return keys;

    }

    public String[] getUpdatedItemKeys(String sessionId, String since,
                                       String until) {

        log.finest("getUpdatedItemKeys method as been invoked. "
                + "Input parameters are: "
                + "SessionId = " + sessionId
                + ", Since = " + since
                + ", Until = " + until);

        String keys[] = getItemKeys(updatedItems);
        log.finest("keys = " + keys + " will be returned.");

        return keys;

    }

    public String[] getDeletedItemKeys(String sessionId, String since,
                                       String until) {

        log.finest("getDeletedItemKeys method as been invoked. "
                + "Input parameters are: "
                + "SessionId = " + sessionId
                + ", Since = " + since
                + ", Until = " + until);

        String keys[] = getItemKeys(deletedItems);
        log.finest("keys = " + keys + " will be returned.");

        return keys;

    }


    public Item addItem(String sessionId, Item item) {

        log.finest("addItem method as been invoked. "
                + "Input parameters are: "
                + " SessionId = " + sessionId
                + ", Item = " + item);

        newItems.put(item.getKey(), item);

        log.finest("item = " + item + " will be returned.");

        return item;

    }

    public Item updateItem(String sessionId, Item item) {

        log.finest("updateItem method as been invoked. "
                + "Input parameters are: "
                + "SessionId = " + sessionId
                + ", Item = " + item);

        updatedItems.put(item.getKey(), item);

        log.finest("item = " + item + " will be returned.");

        return item;

    }

    public void removeItem(String sessionId, String itemKey, String timestamp, boolean softDeletion) {

        Item item = findItem(itemKey);

        if (item != null) {
            removeItem(itemKey);
            deletedItems.put(itemKey, item);
        }
        log.finest("removeItem method as been invoked. "
                + "Input parameters are: "
                + "SessionId = " + sessionId
                + ", ItemKey = " + itemKey
                + ", SoftDeletion = " + softDeletion
                + ", DeletionTimestamp = " + timestamp);

    }

    public void setOperationStatus(String sessionId, String operation, int statusCode, String[] itemKeys) {

        log.finest("setOperationStatus method as been invoked. "
                + "Input parameters are: "
                + "SessionId= " + sessionId
                + ", Operation= " + operation
                + ", Status code= " + statusCode
                + ", ItemKeys= " + itemKeys.toString());

    }

    public byte getItemStateFromId(String sessionId, String itemKey) {

        log.finest("getSyncItemStateFromId method as been invoked. "
                + "Input parameter are: "
                + "SessionId = " + sessionId
                + ", Item key = " + itemKey);

        byte state = getItemState(itemKey);
        log.finest("item state = " + (char) state + " will be returned.");

        return state;

    }

    public boolean isItemInFilterClause(String sessionId, String itemKey) {

        log.finest("isItemInFilterClause method as been invoked. "
                + "Input parameter are: "
                + "SessionId = " + sessionId
                + ", Item key = " + itemKey);

        boolean itemInFilterClause = true;
        log.finest("item is in filter clause");

        return itemInFilterClause;

    }

    public void endSync(String sessionId) {

        saveData();

        log.finest("endSync method as been invoked. "
                + "Input parameter is: "
                + "SessionId = " + sessionId);

    }

    /**
     * todo
     * Merge the server item with the input key with the input client item.
     * @param serverItemKey
     * @param clientItem
     * @return true if the client item is to be updated
     */
    public boolean mergeItems(String serverItemKey, Item clientItem) {
        return true;
    }


    /**
     * todo
     * Get the keys of the input item
     * @param item
     * @return the keys of the input itme
     */
    public String[] getItemKeysFromTwin(Item item) {
        return new String[0];
    }

    //------------------------- Private Methods  --------------------------

    private void createNewSession() {

        id++;
        sessionId = "sid-" + id;

    }

    private Hashtable loadItems(String location) {
        Hashtable table;
        BufferedInputStream fileIn;
        File dir;
        File[] files;
        String fileName;
        StringBuffer fileContent;
        Item item;
        byte[] bytesBuffer;
        int length;

        table = new Hashtable();
        dir = new File(location);
        files = dir.listFiles();
        bytesBuffer = new byte[1024];
        if (files == null) {
            // Either dir does not exist or is not a directory
        } else {
            for (int i = 0; i < files.length; i++) {
                // Get filename of file or directory
                if (files[i].isFile()) {

                    fileContent = new StringBuffer();
                    fileIn = null;
                    try {
                        fileIn = new BufferedInputStream(
                                new FileInputStream(files[i]));

                        while ((length = fileIn.read(bytesBuffer)) > 0) {
                            fileContent.append(new String(bytesBuffer, 0, length));
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        if (fileIn!=null) {
                            try {
                                fileIn.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }

                    fileName = files[i].getName();

                    item = new Item(fileName,
                            format,
                            type,
                            fileContent.toString().getBytes());

                    table.put(fileName, item);

                }
            }

        }
        return table;
    }

    private String[] getItemKeys(Hashtable table) {

        Enumeration keys = table.keys();
        ArrayList list = new ArrayList();

        while (keys.hasMoreElements()) list.add(keys.nextElement());

        return (String[]) list.toArray(new String[0]);

    }

    private String[] getAllItemKeys() {

        Enumeration keys = synchronizedItems.keys();
        ArrayList list = new ArrayList();

        while (keys.hasMoreElements()) list.add(keys.nextElement());

        keys = updatedItems.keys();
        while (keys.hasMoreElements()) list.add(keys.nextElement());

        keys = newItems.keys();
        while (keys.hasMoreElements()) list.add(keys.nextElement());

        return (String[]) list.toArray(new String[0]);

    }

    private Item findItem(String key) {

        Item item = (Item) synchronizedItems.get(key);
        if (item != null)
            return item;

        item = (Item) updatedItems.get(key);
        if (item != null)
            return item;

        item = (Item) newItems.get(key);

        return item;

    }

    private void removeItem(String key) {

        synchronizedItems.remove(key);

        updatedItems.remove(key);

        newItems.remove(key);

        deletedItems.remove(key);

    }

    private byte getItemState(String key) {

        if (deletedItems.get(key) != null)
            return 'D';
        else if (updatedItems.get(key) != null)
            return 'U';
        else if (newItems.get(key) != null)
            return 'N';
        else if (synchronizedItems.get(key) != null)
            return 'S';

        return 'X';

    }


    private void loadConfigurations() {

        Properties props = new Properties();
        try {

            props.load(new FileInputStream(PROPERTIES_FILE_NAME));
        } catch (IOException ex) {
            System.out.println("Error while trying to read configuration properties");
        }

        dbLocation = props.getProperty(PROPERTY_DB_LOCATION, DEFAULD_DB_LOCATION);
        type = props.getProperty(PROPERTY_TYPE, DEFAULD_TYPE);
        format = props.getProperty(PROPERTY_FORMAT, DEFAULD_FORMAT);

    }

    private void saveItems(Hashtable itemsToSave, String location) {


        FileWriter file;
        BufferedWriter writer;

        ArrayList list = new ArrayList();

        Enumeration enumeration = itemsToSave.elements();
        while (enumeration.hasMoreElements()) list.add(enumeration.nextElement());

        Item items[] = (Item[]) list.toArray(new Item[0]);

        for (int i = 0; i < items.length; i++) {

            try {

                file = new FileWriter(location + items[i].getKey());
                writer = new BufferedWriter(file);
                writer.write(new String(items[i].getContent()),
                        0,
                        items[i].getContent().length);
                writer.close();
                file.close();


            } catch (IOException ex) {
                ex.printStackTrace();
            }

        }

        deleteItems(location);

    }

    private void deleteItems(String location) {

        ArrayList list = new ArrayList();

        Enumeration enumeration = deletedItems.elements();
        while (enumeration.hasMoreElements()) list.add(enumeration.nextElement());

        Item items[] = (Item[]) list.toArray(new Item[0]);

        for (int i = 0; i < items.length; i++) {

            new File(location + items[i].getKey()).delete();

        }
    }

    private void deleteItems(Hashtable items, String location) {
        Enumeration itemsEnum;
        itemsEnum = items.elements();

        while(itemsEnum.hasMoreElements()) {
            new File(location+((Item)itemsEnum.nextElement()).getKey()).delete();
        }
    }

    private void loadData() {

        loadConfigurations();
        synchronizedItems = loadItems(dbLocation + SYNCHRONIZED_ITEMS_LOCATION);
        newItems = loadItems(dbLocation + NEW_ITEMS_LOCATION);
        updatedItems = loadItems(dbLocation + UPDATED_ITEMS_LOCATION);
        deletedItems = loadItems(dbLocation + DELETED_ITEMS_LOCATION);

        log.finest("Successfully loaded items " + getAllItemKeys().toString() +
                "from database.");
    }

    private void saveData() {

        checkDbLocation();

        saveItems(synchronizedItems, dbLocation + SYNCHRONIZED_ITEMS_LOCATION);
        saveItems(newItems, dbLocation + NEW_ITEMS_LOCATION);
        deleteItems(updatedItems, dbLocation+SYNCHRONIZED_ITEMS_LOCATION);
        saveItems(updatedItems, dbLocation + UPDATED_ITEMS_LOCATION);
        saveItems(deletedItems, dbLocation + DELETED_ITEMS_LOCATION);

        log.finest("Successfully saved items " + getAllItemKeys().toString() +
                "from database.");
    }

    private void checkDbLocation() {

        // Create a directory; all non-existent ancestor directories are
        // automatically created
        new File(dbLocation).mkdirs();
        new File(dbLocation + SYNCHRONIZED_ITEMS_LOCATION).mkdir();
        new File(dbLocation + NEW_ITEMS_LOCATION).mkdir();
        new File(dbLocation + UPDATED_ITEMS_LOCATION).mkdir();
        new File(dbLocation + DELETED_ITEMS_LOCATION).mkdir();

    }


}
