/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.ws.admin;

import java.io.Serializable;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.framework.engine.source.ContentType;
import com.funambol.framework.engine.source.SyncSourceInfo;

import com.funambol.admin.AdminException;
import com.funambol.admin.ui.SourceManagementPanel;

import com.funambol.ws.engine.source.WebServiceSyncSource;

/**
 * This class implements the configuration panel for an WebServiceSyncSource
 *
 * @version $Id: WebServiceSyncSourceConfigPanel.java,v 1.1 2006/08/08 08:57:21 gibi Exp $
 */
public class WebServiceSyncSourceConfigPanel
extends SourceManagementPanel
implements Serializable {

    // --------------------------------------------------------------- Constants

    /**
     * Allowed characters for name and uri
     */
    public static final String NAME_ALLOWED_CHARS
    = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";

    // ------------------------------------------------------------ Private data
    /** label for the panel's name */
    private JLabel panelName = new JLabel();

    private JLabel           nameLabel          = new JLabel()     ;
    private JTextField       nameValue          = new JTextField() ;
    private JLabel           typeLabel          = new JLabel()     ;
    private JTextField       typeValue          = new JTextField() ;
    private JLabel           sourceUriLabel     = new JLabel()     ;
    private JTextField       sourceUriValue     = new JTextField() ;
    private JLabel           webServiceUrlLabel = new JLabel()     ;
    private JTextField       webServiceUrlValue = new JTextField() ;
    private JLabel           filterLabel        = new JLabel();
    private JCheckBox        filterValue        = new JCheckBox();

    private JButton          confirmButton      = new JButton()    ;


    // ------------------------------------------------------------ Constructors

    /**
     * Creates a new WebServiceSyncSourceConfigPanel instance
     */
    public WebServiceSyncSourceConfigPanel() {
        init();
    }

    // --------------------------------------------------------- Private methods

    /**
     * Create the panel
     */
    private void init(){
        // set layout
        this.setLayout(null);

        // set properties of label, position and border
        //  referred to the title of the panel
        TitledBorder titledBorder1 = new TitledBorder("");

        panelName.setFont(titlePanelFont);
        panelName.setText("Edit Web Service SyncSource");
        panelName.setBounds(new Rectangle(14, 5, 316, 28));
        panelName.setAlignmentX(SwingConstants.CENTER);
        panelName.setBorder(titledBorder1);

        sourceUriLabel.setText("Source URI: ");
        sourceUriLabel.setFont(defaultFont);
        sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 18));
        sourceUriValue.setFont(defaultFont);
        sourceUriValue.setBounds(new Rectangle(170, 60, 350, 18));

        nameLabel.setText("Name: ");
        nameLabel.setFont(defaultFont);
        nameLabel.setBounds(new Rectangle(14, 90, 150, 18));
        nameValue.setFont(defaultFont);
        nameValue.setBounds(new Rectangle(170, 90, 350, 18));

        typeLabel.setText("Type: ");
        typeLabel.setFont(defaultFont);
        typeLabel.setBounds(new Rectangle(14, 120, 150, 18));
        typeValue.setFont(defaultFont);
        typeValue.setBounds(new Rectangle(170, 120, 350, 18));

        webServiceUrlLabel.setText("Web Service URL: ");
        webServiceUrlLabel.setFont(defaultFont);
        webServiceUrlLabel.setBounds(new Rectangle(14, 150, 150, 18));
        webServiceUrlValue.setFont(defaultFont);
        webServiceUrlValue.setBounds(new Rectangle(170, 150, 350, 18));

        filterLabel.setText("Enable filtering: ");
        filterLabel.setFont(defaultFont);
        filterLabel.setBounds(new Rectangle(14, 180, 150, 18));
        filterValue.setSelected(true);
        filterValue.setBounds(new Rectangle(167, 180, 350, 18));

        confirmButton.setFont(defaultFont);
        confirmButton.setText("Add");
        confirmButton.setBounds(170, 230, 70, 25);

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event ) {
                try {
                    validateValues();
                    getValues();
                    if (getState() == STATE_INSERT) {
                        WebServiceSyncSourceConfigPanel.this.actionPerformed(new ActionEvent(WebServiceSyncSourceConfigPanel.this, ACTION_EVENT_INSERT, event.getActionCommand()));
                    } else {
                        WebServiceSyncSourceConfigPanel.this.actionPerformed(new ActionEvent(WebServiceSyncSourceConfigPanel.this, ACTION_EVENT_UPDATE, event.getActionCommand()));
                    }
                } catch (Exception e) {
                    notifyError(new AdminException(e.getMessage()));
                }
            }
        });

        // add all components to the panel
        this.add(panelName         , null);
        this.add(nameLabel         , null);
        this.add(nameValue         , null);
        this.add(typeLabel         , null);
        this.add(typeValue         , null);
        this.add(sourceUriLabel    , null);
        this.add(sourceUriValue    , null);
        this.add(webServiceUrlLabel, null);
        this.add(webServiceUrlValue, null);
        this.add(filterLabel       , null);
        this.add(filterValue       , null);
        this.add(confirmButton     , null);
    }

    /**
     * Loads the given syncSource showing the name, uri, type , web service url
     * and filter type in the panel's
     * fields.
     *
     */
    public void updateForm() {
         if (!(getSyncSource() instanceof WebServiceSyncSource)) {
          notifyError(
              new AdminException(
                  "This is not an WebServiceSyncSource! Unable to process SyncSource values."
              )
          );
          return;
        }

        WebServiceSyncSource syncSource = (WebServiceSyncSource)getSyncSource();

        if (getState() == STATE_INSERT) {
          confirmButton.setText("Add");
        } else if (getState() == STATE_UPDATE) {
          confirmButton.setText("Save");
        }


        sourceUriValue.setText(syncSource.getSourceURI() );
        nameValue.setText     (syncSource.getName()      );
        typeValue.setText         (syncSource.getType()         );
        webServiceUrlValue.setText(syncSource.getWebServiceUrl());
        filterValue.setSelected   (syncSource.isFilter()        );

        if (syncSource.getSourceURI() != null) {
            sourceUriValue.setEditable(false);
        }
    }

    // ----------------------------------------------------------- Private methods
    /**
     * Checks if the values provided by the user are all valid. In caso of errors,
     * a IllegalArgumentException is thrown.
     *
     * @throws IllegalArgumentException if:
     *         <ul>
     *         <li>name, uri, type, web service, filter type or directory are
     *         <li>empty (null or zero-length)
     *         <li>the types list length does not match the versions list length
     *         </ul>
     */
    private void validateValues() throws IllegalArgumentException {
        String value;

        value = nameValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Name' cannot be empty. Please provide a SyncSource name.");
        }

        if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
            throw new
            IllegalArgumentException(
            "Only the following characters are allowed for field 'Name': \n" + NAME_ALLOWED_CHARS);
        }

        value = typeValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Type' cannot be empty. Please provide a SyncSource type.");
        }

        value = sourceUriValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Source URI' cannot be empty. Please provide a SyncSource URI.");
        }

        value = webServiceUrlValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Web Service URL' cannot be empty. Please provide a Web Service URL.");
        }
    }

    /**
     * Set syncSource properties with the values provided by the user.
     */
    private void getValues() {

        WebServiceSyncSource syncSource = (WebServiceSyncSource)getSyncSource();

        syncSource.setSourceURI      (sourceUriValue.getText().trim());
        syncSource.setName           (nameValue.getText().trim()     );
        syncSource.setType           (typeValue.getText().trim()     );
        syncSource.setWebServiceUrl  (webServiceUrlValue.getText().trim()     );
        syncSource.setFilter         (filterValue.isSelected());

        ContentType[] contentTypes = new ContentType[] {
            new ContentType("text/plain", "1.0")
        };

        syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));
    }
}
