/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.ws.client;

import java.security.Principal;

import java.sql.Timestamp;

import java.text.SimpleDateFormat;

import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.filter.FilterClause;
import com.funambol.framework.security.Sync4jPrincipal;

import com.funambol.ws.engine.source.WebServiceSyncSource;

/**
 * Manages the business delegate access
 *
 * @version $Id: WebServiceSyncSourceController.java,v 1.1 2006/08/08 08:58:12 gibi Exp $
 */
public class WebServiceSyncSourceController implements java.io.Serializable {

    // ------------------------------------------------------------ Private data

    /** The web service sync source */
    private WebServiceSyncSource syncSource = null;


    public WebServiceSyncSource getWebServiceSyncSource() {
        return syncSource;
    }

    /** BusinessDelegate to access the business methods */
    private BusinessDelegate businessDelegate = null;

    /** Returns BusinessDelegate */
    public BusinessDelegate getBusinessDelegate() {
        return businessDelegate;
    }

    /** The session id created in the web service for this sync source */
    private String sessionId = null;

    /** Returns the session id */
    public String getSessionId() {
        return sessionId;
    }

    private void setSessionId(String id) {
        sessionId = id;
    }

    // ---------------------------------------------------------- Constructor

    /**
     * Called to create main resource
     */
    public WebServiceSyncSourceController(WebServiceSyncSource syncSource)
    throws SyncSourceException{

        this.syncSource = syncSource;

        businessDelegate = new BusinessDelegate(syncSource);


        }

    // ---------------------------------------------------------- Public Methods



    /** controlls the beginSync process */
    public void beginSync(Principal principal, int syncMode
                                             , FilterClause filterClause)
    throws SyncSourceException {

        long principalId = ((Sync4jPrincipal)principal).getId();

        setSessionId(businessDelegate.beginSync(new Long(principalId),
                                                new Integer(syncMode),
                                                filterClause));
        }

    /** controlls the beginSync process without the filter*/
    public void beginSync(Principal principal, int syncMode)
    throws SyncSourceException {

        long principalId = ((Sync4jPrincipal)principal).getId();

        setSessionId(businessDelegate.beginSync(new Long(principalId),
                                                new Integer(syncMode)));
    }

    /** controlls the getItemFromId process */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
    throws SyncSourceException {


            String itemKey = syncItemKey.getKeyAsString();

            Item item = businessDelegate.getItemFromId(sessionId, itemKey);

            return itemToSyncItem(item);

    }

    /** controlls the getAllItemKeys process */
    public SyncItemKey[] getAllSyncItemKeys()
    throws SyncSourceException {

            String itemKeys[] = businessDelegate.getAllItemKeys(sessionId);

            return itemKeysToSyncItemKeys(itemKeys);

    }

    /** controlls the getNewItemKeys process */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {

            String sinceTs = timestampToString(since);
            String untilTs = timestampToString(until);

            String itemKeys[] = businessDelegate.getNewItemKeys(sessionId
                                                               ,sinceTs
                                                               ,untilTs);

            return itemKeysToSyncItemKeys(itemKeys);


    }

    /** controlls the getUpdatedItemKeys process */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {


            String sinceTs = timestampToString(since);
            String untilTs = timestampToString(until);

            String itemKeys[] = businessDelegate.getUpdatedItemKeys(sessionId
                                                                   ,sinceTs
                                                                   ,untilTs);

            return itemKeysToSyncItemKeys(itemKeys);


    }

    /** controlls the getDeletedItemKeys process */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {

            String sinceTs = timestampToString(since);
            String untilTs = timestampToString(until);

            String itemKeys[] = businessDelegate.getDeletedItemKeys(sessionId
                                                                   ,sinceTs
                                                                   ,untilTs);
            return itemKeysToSyncItemKeys(itemKeys);


    }

    /** controlls the addItem process */
    public SyncItem addSyncItem(SyncItem syncItem)
    throws SyncSourceException {

            Item item = businessDelegate.addItem(sessionId, syncItemToItem(syncItem));

            return itemToSyncItem(item);

    }

    /** controlls the updateItem process */
    public SyncItem updateSyncItem(SyncItem syncItem)
    throws SyncSourceException {

            Item item = businessDelegate.updateItem(sessionId, syncItemToItem(syncItem));

            return itemToSyncItem(item);

    }

    /** controlls the removeItem process */
    public void removeSyncItem(SyncItemKey syncItemKey, Timestamp deletionTime, boolean softDeletion)
    throws SyncSourceException {


            String itemKey = syncItemKey.getKeyAsString();

            String deletionTimeTs = timestampToString(deletionTime);

            businessDelegate.removeItem(sessionId, itemKey, deletionTimeTs, softDeletion);


    }

    /** controlls the endSync process */
    public void endSync() throws SyncSourceException {

            businessDelegate.endSync(sessionId);

    }

    /** controlls the set operation status process */
    public void setOperationStatus(String operation, int statusCode, SyncItemKey[] keys)
    throws SyncSourceException{

            String[] itemKeys = new String[keys.length];
            for (int i = 0; i < itemKeys.length; i++) {

                itemKeys[i] = keys[i].getKeyAsString();

            }

            businessDelegate.setOperationStatus(sessionId, operation, statusCode, itemKeys);

    }

    /** controlls the getSyncItemStateFromId process */
    public char getSyncItemStateFromId(SyncItemKey key)
    throws SyncSourceException {

        return businessDelegate.getItemStateFromId(sessionId,
                                                   key.getKeyAsString());

    }

    /** controlls the isSyncItemInFilterClause process */
    public boolean isSyncItemInFilterClause(SyncItemKey key)
    throws SyncSourceException {

        return businessDelegate.isItemInFilterClause(sessionId,
                                                     key.getKeyAsString());

    }

    /**
     * Controls the mergeItems process
     * @param serverSyncItemKey
     * @param clientSyncItem
     * @return true is the client items is to be updated, false otherwise
     * @throws SyncSourceException
     */
    public boolean mergeSyncItems(SyncItemKey serverSyncItemKey, SyncItem clientSyncItem)
            throws SyncSourceException {

        String serverItemKey;
        Item clientItem;

        serverItemKey = serverSyncItemKey.getKeyAsString();
        clientItem = new Item(clientSyncItem.getKey().getKeyAsString(),
                clientSyncItem.getFormat(),
                clientSyncItem.getType(),
                clientSyncItem.getContent());

        return businessDelegate.mergeItems(serverItemKey, clientItem);
    }

    /**
     * Controls the getSyncItemFromTwin process
     * @param syncItem
     * @return an array with the twin sync items's keys
     * @throws SyncSourceException
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem)
    throws SyncSourceException {
        String[] twinItemKeys;

        twinItemKeys = businessDelegate.getItemKeysFromTwin(syncItemToItem(syncItem));

        return itemKeysToSyncItemKeys(twinItemKeys);
    }

    // --------------------------------------------------------- Private methods

    /** Convert one item into a sync item */
    private SyncItem itemToSyncItem(Item item) {

        //TO Compleet
        SyncItem syncItem = new SyncItemImpl(syncSource, item.getKey());

        if(item.getFormat() != null)
            syncItem.setFormat(item.getFormat());

        if(item.getType() != null)
            syncItem.setType(item.getType());

        if(item.getContent() != null)
            syncItem.setContent(item.getContent());

        return syncItem;

    }

    /** Convert one sync item into a item */
    private Item syncItemToItem(SyncItem syncItem) {

        //To Compleet
        Item item = new Item();
        item.setKey(syncItem.getKey().getKeyAsString());

        item.setFormat(syncItem.getFormat());

        item.setContent(syncItem.getContent());
        return item;

    }

    /** Convert string keys into sync item keys */
    private SyncItemKey[] itemKeysToSyncItemKeys(String itemKey[]) {

            SyncItemKey syncItemKeys[] = new SyncItemKey[itemKey.length];

            for(int i = 0; i < itemKey.length; i++)
                syncItemKeys[i] = new SyncItemKey(itemKey[i]);

            return syncItemKeys;

    }

    /** Convert Timestamps into a formated string with the folowing pattern UTC
     *  yyyyMMdd'T'HHmmss'Z'
     */
    private String timestampToString(Timestamp timestamp) {

        SimpleDateFormat formated =
                (SimpleDateFormat)SimpleDateFormat.getDateInstance();
        formated.applyPattern("yyyyMMdd'T'HHmmss'Z'");

        return formated.format(timestamp);
    }
}
